#include <iostream>
#include <cstdlib>
#include <cstring>
using namespace std;

class IntegerArray {

public:
	explicit IntegerArray (const int* array = nullptr) {
		if (array == nullptr) return;
		memcpy(int_array, array, Dim * sizeof(int));
	}

	virtual ~IntegerArray() = default;

	virtual int& operator[](int index) {
		return int_array[index];
	}

	ostream& print(ostream& out) const {
		out << "< ";
		for(int e : int_array)
			out << e << " ";
		return out << ">";
	}

protected:
	static constexpr int Dim { 5 };
	int int_array[Dim]{};
};

ostream& operator<< (ostream& out, const IntegerArray& array) {
	return array.print(out);
}

class SafeArray : public IntegerArray {
public:
	explicit SafeArray (const int* array = nullptr) : IntegerArray(array) {}
	int& operator[] (int index) override {
		if (index < 0 || index >= Dim) {
			cout << "Index out of bounds " << index << endl;
			exit(1);
		}
		return int_array[index];
	}
};

void print (IntegerArray& array, int index, const char* name) {
	cout << name << "[" << index << "] = " << array[index] << endl;
}

int main () {
	int array[] = {5, 3, 0, 7, 1};

	IntegerArray int_array(array);
	SafeArray safe_array(array);

	cout << "int_array " << int_array << endl;
	cout << "safe_array " << safe_array << endl;
	cout << endl;

	print(int_array, 0, "int_array");
	print(safe_array, 0, "safe_array");
	print(int_array, 4, "int_array");
	print(safe_array, 4, "safe_array");
	cout << endl;

	print(int_array, 5, "int_array");
	print(safe_array, 5, "safe_array");
	cout << endl;

	return 0;
}

